﻿;-------------------------------------------------------------------------------
; Anagrams.ahk
; by wolf_II
;-------------------------------------------------------------------------------
; simple Anagram searcher
; written with massive help by Helgef from AHK forum
; https://autohotkey.com/boards/viewtopic.php?p=158284#p158284
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
AutoExecute: ; auto-execute section of the script
;-------------------------------------------------------------------------------
    #NoEnv                              ; don't check empty variables
    #SingleInstance, Force              ; kill old instance
    ;---------------------------------------------------------------------------
    SetBatchLines, -1                   ; run script at maximum speed
    SetWorkingDir, %A_ScriptDir%        ; consistent directory
    SplitPath, A_ScriptName,,,, AppName ; get AppName from file name
    ;---------------------------------------------------------------------------
    Version := "2.00"
    ;===========================================================================

    Max_Input_Length := 26

    ; display GUI while loading
    Try Menu, Tray, Icon, %AppName%.ico
    Menu, Tray, Icon ; show
    Gosub, MainGuiCreate
    Gui, Show

    ; load word list
    WordCount := Make_DICT()

    ; loading is done
    SB_SetText("Words: " prettify_Number(WordCount), 3)
    GuiControl, Hide, Loading
    GuiControl, -Center -Disabled +gShowAnagrams, Input
    GuiControl, Choose, Skip, 5
    GuiControl, Focus, Input
    GuiControl,, Input

Return ; end of auto-execute section



MainGuiClose:
ExitApp



;-------------------------------------------------------------------------------
#Include, *i %A_ScriptDir%\Include ; set folder for includes
;-------------------------------------------------------------------------------
    #Include, GUI.Create.ahk
    #Include, MainCode.ahk
    #Include, Make_DICT.ahk



;-------------------------------------------------------------------------------
; Library files
;-------------------------------------------------------------------------------
    #Include, <prettify_Number>
    #Include, <QPC>



;-------------------------------------------------------------------------------
ShowAnagrams: ; live display all anagrams of Input found in Dictionary
;-------------------------------------------------------------------------------
    GuiControlGet, Input
    GuiControlGet, Skip
    GuiControlGet, LBox
    InputLength := StrLen(Input)
    SB_SetText("Length: " InputLength, 1)

    If (InputLength > 12) { ; reduce flashing
        GuiControl, +Disabled, Input
        GuiControl, +Disabled, Skip
        GuiControl, +Disabled, LBox
    }

    If (InputLength > Max_Input_Length) {
        GuiControl,, LBox, |Input too long
        GuiControl, -Disabled, Input
        GuiControl, -Disabled, Skip
        GuiControl, -Disabled, LBox
        GuiControl, Focus, Input
        Return
    }

    WordList := "", Count := 0
    Start := QPC()
    For each, Keyword in Combinations(Input, Skip)
        For each, Anagram in DICT[StrLen(Keyword) // 2 + 1, Keyword]
            WordList .= WordList ? "|" Anagram : Anagram, Count++
    Sort, WordList, D| F LengthSort
    TimeTaken := QPC() - Start ; time in s

    GuiControl,, TimeTaken, %TimeTaken%
    GuiControl, Hide, LBox
    GuiControl,, LBox, % "|" (WordList ? WordList : "no anagrams for " Input)
    GuiControl, Show, LBox
    SB_SetText("Count: " Count, 2)
    SB_SetText("Time: " Round(TimeTaken, 3) " s", 4)
    GuiControl, -Disabled, Input
    GuiControl, -Disabled, Skip
    GuiControl, -Disabled, LBox
    GuiControl, Focus, Input

Return



/*--------- end of file --------------------------------------------------------
