﻿


;-------------------------------------------------------------------------------
Make_DICT() { ; make DICT object from text file
;-------------------------------------------------------------------------------
    ; DICT object is global, WordCount gets returned
    ;---------------------------------------------------------------------------
    global DICT := []

    FileRead, Dictionary, Words.txt
    StrReplace(Dictionary, "`n",, WordCount), WordCount++
    WA := A_IsUnicode ? 2 : 1

    Loop, Parse, Dictionary, `n, `r
    {
        If (Percent := Round(100 * A_Index / WordCount)) > Prev_Percent {
            GuiControl,, Loading, %Percent%
            GuiControl,, Input, %Percent% `%
        }
        Prev_Percent := Percent

        WordLength := StrLen(A_LoopField)
        If Not IsObject(DICT[WordLength])
            DICT[WordLength] := []

        ; allow Bob, English and I in Words.txt
        StringLower, Keyword, A_LoopField
        Keyword := make_Keyword(Keyword)
        If Not IsObject(DICT[WordLength, Keyword])
            DICT[WordLength, Keyword] := []

        ;-----------------------------------------------------------------------
        ; Each word in Dictionary gets pushed into the respective DICT[L, K].
        ; Push() function returns the index of the most recently pushed value.
        ; We then use this index to correct the size downwards,
        ; i.e. save space or adjust the capacity of a field in DICT[L, K].
        ;-----------------------------------------------------------------------
        ; E.g. let's say we pushed the 7th word to DICT[L, K] not knowing the 7.
        ; Push() gives us back the index = 7, and we can adjust the 7th field
        ; of DICT[L, K] to ByteSize = # characters (respecting ANSI/Unicode)
        ;-----------------------------------------------------------------------
        ; We do this exercise on each word in Dictionary,
        ; impact on performance is negligible, but:
        ; impact on memory used is noticeable (129 MB vs. 96 MB) on 370 k words
        ; impact on memory used is noticeable ( 40 MB vs. 30 MB) on 110 k words
        ;-----------------------------------------------------------------------
        index := DICT[WordLength, Keyword].Push(A_LoopField)
        DICT[WordLength, Keyword].SetCapacity(index, (WordLength + 1) * WA)
    }

    ; save more space
    For each, WordLength_Obj in DICT
        WordLength_Obj.SetCapacity(0)

    Return, WordCount
}
